using Microsoft.Extensions.Logging.Console;
using MySql.Data.MySqlClient;
using SinMaiLauncher.ChildProcessHolder;
using SinMaiLauncher.Interops;
using SinMaiLauncher.Models;

namespace SinMaiLauncher;

public partial class MainForm : Form
{
    private ILogger logger;

    private readonly IEnumerator<string> txtAni = TextAnimation.LoadingTextSeq(8).GetEnumerator();

    private SettingReader setting;

    private Dictionary<ChildProcessKind, ChildProcessControlGroup> childProcessControlGroups;

    private string mainControlStatusText = "";
    private bool mainControlStatusTextAnimation = false;

    public MainForm()
    {
        InitializeComponent();
    }

    private async void MainForm_Shown(object sender, EventArgs e)
    {
        TopMost = false;
        TopMost = true;

        tcMain.SelectedTab = tpConsoles;

        ConsoleInterops.CurrentProcess.ConsoleAllocate();
        ConsoleInterops.CurrentProcess.ConsoleDisableQuickEdit();
        ConsoleInterops.CurrentProcess.ConsoleWindowSetParent(tpConsoleLauncher);
        ConsoleInterops.CurrentProcess.ConsoleWindowFill(tpConsoleLauncher);
        tpConsoleLauncher.Resize += delegate { ConsoleInterops.CurrentProcess.ConsoleWindowFill(tpConsoleLauncher); };

        //start host application
        var builder = Host.CreateDefaultBuilder();

        builder.ConfigureServices((ctx, services) =>
        {
            //̨־ʽ
            services.AddLogging(opt =>
            {
                opt.AddSimpleConsole(p =>
                    {
                        p.TimestampFormat = "[dd HH:mm:ss] ";
                        p.SingleLine = true;
                        p.ColorBehavior = LoggerColorBehavior.Enabled;
                    })
                    .AddDebug()
                    //.AddProvider(loggingIntoEventBus)
                    ;

                services.AddSingleton(opt);
            });

            services.AddSingleton(this);
        });

        var host = builder.Build();
        var services = host.Services;
        host.Start();

        logger = services.GetRequiredService<ILogger<MainForm>>();
        var conf = services.GetRequiredService<IConfiguration>();

        var settingModel = new SinMaiLauncherSettingModel();
        conf.GetSection("SinMaiLauncher").Bind(settingModel);
        setting = new(settingModel);

        childProcessControlGroups = new Dictionary<ChildProcessKind, ChildProcessControlGroup>
        {
            {
                ChildProcessKind.MariaDb, new ChildProcessControlGroup
                {
                    StateBag = new ProcessStateBagMariaDb(services.GetRequiredService<ILogger<ProcessStateBagMariaDb>>(), setting),
                    ConsoleTab = tpConsoleMaria, PidLabel = lblPidMaria, StatusLabel = lblSubControlMaria,
                    StartButton = btnSubControlMariaStart, StopButton = btnSubControlMariaStop
                }
            },
            {
                ChildProcessKind.AquaDx, new ChildProcessControlGroup
                {
                    StateBag = new ProcessStateBagAquaDx(services.GetRequiredService<ILogger<ProcessStateBagAquaDx>>(), setting),
                    Dep = ChildProcessKind.MariaDb,
                    ConsoleTab = tpConsoleAquaDx, PidLabel = lblPidAquaDx, StatusLabel = lblSubControlAquaDx,
                    StartButton = btnSubControlAquaDxStart, StopButton = btnSubControlAquaDxStop
                }
            },
            {
                ChildProcessKind.Injector, new ChildProcessControlGroup
                {
                    StateBag = new ChildProcessStateBagForConsole(ChildProcessKind.Injector, setting.Injector),
                    Dep = ChildProcessKind.AquaDx,
                    ConsoleTab = tpConsoleInjector, PidLabel = lblPidInjector, StatusLabel = lblSubControlInjector,
                    StartButton = btnSubControlInjectorStart, StopButton = btnSubControlInjectorStop
                }
            },
            {
                ChildProcessKind.SinMai, new ChildProcessControlGroup
                {
                    StateBag = new ChildProcessStateBagForWin(ChildProcessKind.SinMai, setting.SinMai),
                    Dep = ChildProcessKind.Injector,
                    ConsoleTab = null, PidLabel = lblPidSinMai, StatusLabel = lblSubControlSinMai,
                    StartButton = btnSubControlSinMaiStart, StopButton = btnSubControlSinMaiStop
                }
            }
        };

        foreach (var gr in childProcessControlGroups.Values)
        {
            var tab = gr.ConsoleTab;
            var cpb = gr.StateBag;

            var btnStart = gr.StartButton;
            var btnStop = gr.StopButton;

            var lblPid = gr.PidLabel;
            var lblStatus = gr.StatusLabel;

            //tab̨
            if (cpb is ChildProcessStateBagForConsole con)
            {
                tab.Resize += delegate
                {
                    if (con.HWndConsole.HasValue) WindowInterops.FillWindow(con.HWndConsole.Value, tab);
                };
            }

            btnStart.Click += async delegate
            {
                if (cpb.IsAlive)
                {
                    MessageBox.Show("ˣظ");
                    return;
                }

                if (gr.Dep.HasValue && childProcessControlGroups[gr.Dep.Value].StateBag.Status != ChildProcessStatus.Ready)
                {
                    MessageBox.Show("Ľδ˵ģȾҪ˳");
                    return;
                }

                await cpb.StartAsync();
            };

            btnStop.Click += async delegate
            {
                if (cpb.IsAlive == false)
                {
                    MessageBox.Show("ûզֹͣ");
                    return;
                }
                await cpb.StopAsync(TimeSpan.FromSeconds(5));
            };

            cpb.StatusUpdated += async delegate
            {
                void UpdateUi()
                {
                    lblPid.Text = cpb.Pid.HasValue ? cpb.Pid.Value.ToString() : "-";

                    lblStatus.Text = cpb.Status switch
                    {
                        ChildProcessStatus.NoLaunched => "δ",
                        ChildProcessStatus.PreLaunching => "׼",
                        ChildProcessStatus.MissingWorkingDir => "ʧĿ¼",
                        ChildProcessStatus.MissingExeFile => "ʧexeļ",
                        ChildProcessStatus.Launching => "",
                        ChildProcessStatus.WaitingReady => "ȴ...",
                        ChildProcessStatus.Ready => "",
                        ChildProcessStatus.Stopping => "ֹͣ",
                        ChildProcessStatus.Stopped => "ֹͣ",
                        _ => "?"
                    };

                    if (cpb.Status == ChildProcessStatus.WaitingReady && cpb is ChildProcessStateBagForConsole con && cpb.IsAlive && cpb.Pid.HasValue && con.HWndConsole.HasValue)
                    {
                        if (tab != null)
                        {
                            //console 
                            Program.DisableQuickEdit(cpb.Pid.Value);
                            WindowInterops.RemoveBorderVisible(con.HWndConsole.Value);
                            WindowInterops.SetParent(con.HWndConsole.Value, tab.Handle);
                            WindowInterops.FillWindow(con.HWndConsole.Value, tab);
                            Height++;
                            Height--;
                        }
                    }

                    if (cpb.Status == ChildProcessStatus.Ready && cpb is ChildProcessStateBagForWin win)
                    {
                        //chkExplorerAuto
                    }
                }

                if (InvokeRequired) await InvokeAsync(UpdateUi);
                else UpdateUi();
            };
        }

        logger.LogInformation("Started");

        Application.DoEvents();
        await Task.Delay(1000);

        tcMain.Enabled = true;

        //#if DEBUG
        //        tcMain.SelectedTab = tpSubControls;
        //#else
        tcMain.SelectedTab = tpMainControl;
        //#endif
    }

    private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
    {
        if (
            childProcessControlGroups[ChildProcessKind.MariaDb].StateBag.Status == ChildProcessStatus.Ready
            || childProcessControlGroups[ChildProcessKind.AquaDx].StateBag.Status == ChildProcessStatus.Ready
        )
        {
            e.Cancel = MessageBox.Show($"ʩС{Environment.NewLine}Ϊ˱ѿȷȫֹͣٹرա{Environment.NewLine}Ҫǿƹر", "ر", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == DialogResult.No;
        }
    }

    private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
    {
        //Kill all child process
        childProcessControlGroups[ChildProcessKind.SinMai].StateBag.StopAsync(TimeSpan.FromSeconds(10)).Wait(TimeSpan.FromSeconds(10));
        childProcessControlGroups[ChildProcessKind.Injector].StateBag.StopAsync(TimeSpan.FromSeconds(10)).Wait(TimeSpan.FromSeconds(10));
        childProcessControlGroups[ChildProcessKind.AquaDx].StateBag.StopAsync(TimeSpan.FromSeconds(10)).Wait(TimeSpan.FromSeconds(10));
        childProcessControlGroups[ChildProcessKind.MariaDb].StateBag.StopAsync(TimeSpan.FromSeconds(10)).Wait(TimeSpan.FromSeconds(10));
    }

    private void tcConsoles_SelectedIndexChanged(object sender, EventArgs e)
    {
        Height++;
        Height--;
    }

    private class ChildProcessControlGroup
    {
        public ChildProcessStateBag StateBag { get; set; }
        public ChildProcessKind? Dep { get; set; }
        public TabPage? ConsoleTab { get; set; } // ɿգڽΪ null
        public Label PidLabel { get; set; }
        public Label StatusLabel { get; set; }
        public Button StartButton { get; set; }
        public Button StopButton { get; set; }
    }

    private async void btnMainControlStart_Click(object sender, EventArgs e)
    {
        btnMainControlStart.Enabled = false;
        mainControlStatusTextAnimation = true;

        var orderedProcesses = new List<(ChildProcessKind kind, TimeSpan timeout)>
        {
            (ChildProcessKind.MariaDb, TimeSpan.FromSeconds(30)),
            (ChildProcessKind.AquaDx,TimeSpan.FromSeconds(90)),
            (ChildProcessKind.Injector,TimeSpan.FromSeconds(10)),
            (ChildProcessKind.SinMai,TimeSpan.FromSeconds(10)),
        };

        var startedProcesses = new List<ChildProcessKind>();
        try
        {
            foreach (var (kind, timeout) in orderedProcesses)
            {
                var gr = childProcessControlGroups[kind];
                var cpb = gr.StateBag;

                if (cpb.IsAlive)
                {
                    logger.LogWarning("{Kind} is already running, skipping.", kind);
                    startedProcesses.Add(kind); // еҲ¼ڻع
                    continue;
                }

                mainControlStatusText = $" {kind}";

                await cpb.StartAsync();
                await WaitForReadyAsync(cpb, timeout);

                if (cpb.Status != ChildProcessStatus.Ready)
                {
                    logger.LogError($"{kind} failed to reach Ready state (current: {cpb.Status}), rolling back.");
                    await RollbackAsync(startedProcesses);
                    return;
                }
                startedProcesses.Add(kind);
            }

            mainControlStatusText = $"";
            btnMainControlStop.Enabled = true;
            mainControlStatusTextAnimation = false;
        }
        catch (Exception exception)
        {
            logger.LogError(exception, "Error during startup, rolling back.");
            await RollbackAsync(startedProcesses);
        }
        async Task WaitForReadyAsync(ChildProcessStateBag cpb, TimeSpan timeout)
        {
            if (cpb.Status == ChildProcessStatus.Ready
                || cpb.Status == ChildProcessStatus.MissingWorkingDir
                || cpb.Status == ChildProcessStatus.MissingExeFile
                || cpb.Status == ChildProcessStatus.Stopped) return;

            var tcs = new TaskCompletionSource<bool>();
            EventHandler<EventArgs> handler = null;

            handler = delegate
            {
                if (cpb.Status == ChildProcessStatus.Ready
                    || cpb.Status == ChildProcessStatus.MissingWorkingDir
                    || cpb.Status == ChildProcessStatus.MissingExeFile
                    || cpb.Status == ChildProcessStatus.Stopped
                   )
                {
                    cpb.StatusUpdated -= handler;
                    tcs.TrySetResult(true);
                }
            };
            cpb.StatusUpdated += handler;

            try
            {
                // ȴ Ready ʱ
                await Task.WhenAny(tcs.Task, Task.Delay(timeout));
                if (!tcs.Task.IsCompleted)
                {
                    cpb.StatusUpdated -= handler;
                    throw new TimeoutException($"{cpb.Kind} did not reach Ready state within {timeout.TotalSeconds} seconds.");
                }
            }
            finally
            {
                cpb.StatusUpdated -= handler; // ȷ
            }
        }

        async Task RollbackAsync(List<ChildProcessKind> startedProcesses)
        {
            // ෴˳ֹͣ
            foreach (var kind in startedProcesses.AsEnumerable().Reverse())
            {
                mainControlStatusText = $"ֹͣ {kind}";

                var group = childProcessControlGroups[kind];
                var cpb = group.StateBag;

                if (cpb.IsAlive)
                {
                    logger.LogInformation("Stopping {Kind} during rollback...", kind);
                    await InvokeAsync(() => group.StatusLabel.Text = "ֹͣ...");
                    await cpb.StopAsync(TimeSpan.FromSeconds(5));
                }
            }

            mainControlStatusText = "ʧܣѻع֮ǰ";
            btnMainControlStart.Enabled = true;
            mainControlStatusTextAnimation = false;
        }
    }

    private async void btnMainControlStop_Click(object sender, EventArgs e)
    {
        var orderedProcesses = new List<(ChildProcessKind kind, TimeSpan timeout)>
        {
            (ChildProcessKind.SinMai,TimeSpan.FromSeconds(5)),
            (ChildProcessKind.Injector,TimeSpan.FromSeconds(10)),
            (ChildProcessKind.AquaDx,TimeSpan.FromSeconds(90)),
            (ChildProcessKind.MariaDb, TimeSpan.FromSeconds(30)),
        };

        foreach (var (kind, timeout) in orderedProcesses)
        {
            mainControlStatusText = $"ֹͣ {kind}";
            var gr = childProcessControlGroups[kind];

            if (gr.StateBag.IsAlive)
            {
                await gr.StateBag.StopAsync(timeout);
            }
        }

        btnMainControlStart.Enabled = true;
        btnMainControlStop.Enabled = false;
        mainControlStatusText = $"ֹͣ";
        mainControlStatusTextAnimation = false;
    }

    private DateTime lastCheckWin = DateTime.Now;

    private void tmrMain_Tick(object sender, EventArgs e)
    {
        if (mainControlStatusTextAnimation)
        {
            txtAni.MoveNext();
            lblMainControlStatus.Text = $"{mainControlStatusText} {txtAni.Current}";
        }
        else
        {
            lblMainControlStatus.Text = $"{mainControlStatusText}";
        }

        //ڵ
        if (chkWinTrickAuto.Checked && lastCheckWin < DateTime.Now.AddSeconds(-5))
        {
            lastCheckWin = DateTime.Now;

            var pb = (ChildProcessStateBagForWin)childProcessControlGroups[ChildProcessKind.SinMai].StateBag;

            if (pb.Status == ChildProcessStatus.Ready && pb.HWndMainWindow.HasValue)
            {
                MoveSinMaiMainWindow();
            }
        }
    }

    private void btnAimeWrite_Click(object sender, EventArgs e)
    {
        var input = cboAimeCard.Text;

        if (string.IsNullOrEmpty(input))
        {
            MessageBox.Show("aimeŰ");
            return;
        }

        if (Directory.Exists(setting.AimeFleSinMaiDir) == false)
        {
            MessageBox.Show("ҲдaimeļĿ¼");
            return;
        }

        if (Directory.Exists(setting.AimeFleSinMaiPath))
        {
            MessageBox.Show("Ҳдaimeļ");
            return;
        }

        var aime = input.Replace("-", "");

        File.WriteAllText(setting.AimeFleSinMaiPath, aime);
    }

    private void btnWinTrick_Click(object sender, EventArgs e)
    {
        var pb = (ChildProcessStateBagForWin)childProcessControlGroups[ChildProcessKind.SinMai].StateBag;

        if (pb.Status != ChildProcessStatus.Ready || !pb.HWndMainWindow.HasValue)
        {
            MessageBox.Show("Ϸ֮ٵť");
            return;
        }

        var moveWindowResult = MoveSinMaiMainWindow();
        switch (moveWindowResult)
        {
            case WindowInterops.MoveWindowResult.FailGet:
                MessageBox.Show("ɹȡʧ");
                break;

            case WindowInterops.MoveWindowResult.FailSet:
                MessageBox.Show("ɹʧ");
                break;

            case WindowInterops.MoveWindowResult.Success:
                MessageBox.Show("ɹ");
                break;

            case WindowInterops.MoveWindowResult.NoChange:
                MessageBox.Show("ûиı");
                break;
        }
    }

    private WindowInterops.MoveWindowResult MoveSinMaiMainWindow()
    {
        var pb = (ChildProcessStateBagForWin)childProcessControlGroups[ChildProcessKind.SinMai].StateBag;

        //WS_CLIPSIBLINGS WS_POPUPWINDOW WS_VISIBLE

        var rc = setting.Settings.AutoRect;
        return WindowInterops.MoveWindow(pb.HWndMainWindow.Value, rc.Left, rc.Top, rc.Width, rc.Height);
    }

    private void btnAimeRefresh_Click(object sender, EventArgs e)
    {
        if (childProcessControlGroups[ChildProcessKind.MariaDb].StateBag.Status != ChildProcessStatus.Ready)
        {
            MessageBox.Show("ݿȰ");
            return;
        }

        var conf = setting.Settings.Infra.MariaDb;

        var connectionString = $"Server={conf.Host};Port={conf.Port};Uid={conf.Usr};Pwd={conf.Pwd};";

        using var conn = new MySqlConnection(connectionString);
        conn.Open();
        using var cmd = conn.CreateCommand();
        cmd.CommandText = "SELECT `luid` FROM `aqua`.`sega_card`";
        using var dr = cmd.ExecuteReader();
        cboAimeCard.Items.Clear();

        while (dr.Read())
        {
            var aimeR = dr["luid"].ToString();
            var aimeH = string.Join("-",
                aimeR.Substring(0, 4),
                aimeR.Substring(4, 4),
                aimeR.Substring(8, 4),
                aimeR.Substring(12, 4),
                aimeR.Substring(16, 4));

            cboAimeCard.Items.Add(aimeH);
        }

        MessageBox.Show($"ݿжȡ {cboAimeCard.Items.Count} б");
    }

    private void btnExplorerKill_Click(object sender, EventArgs e)
    {
        MessageBox.Show("û");
    }

    private void btnExplorerRestore_Click(object sender, EventArgs e)
    {
        MessageBox.Show("û");
    }

    private void btnProcessLoopBackVirtualNic_Click(object sender, EventArgs e)
    {
        MessageBox.Show("û");
    }

    private void btnAppDataOverwrite_Click(object sender, EventArgs e)
    {
        MessageBox.Show("û");
    }
}